# type: ignore

import os
import re
import sys
import mediacollectiongrooming
import xml.dom.minidom

import typing


def find_videos(path):
    a = mediacollectiongrooming.walk_tree(path)
    b = mediacollectiongrooming.filter_video(a)
    c = mediacollectiongrooming.filter_inaccessible(b)
    d = c
    e = mediacollectiongrooming.absolutize_paths(d)
    f = mediacollectiongrooming.video_to_videoandnfo(e)
    return f


def donfo(video, nfo, dryrun):
    # <musicvideo>
    #   <title>Bestsongintheworld</title>
    #   <artist>Bestartistintheworld</artist>
    #   <album>Me</album>
    #   <genre>Pop</genre>
    #   <runtime>3:20</runtime>
    #   <plot>Scantly clad women hoing about</plot>
    #   <year>2000</year>
    #   <director>and I</director>
    #   <studio>Ego prod.</studio>
    #   <track>1</track>
    # </musicvideo>
    d = xml.dom.minidom.parseString("<musicvideo></musicvideo>")
    musicvideoelement = d.getElementsByTagName("musicvideo")[0]

    def setElement(name, value):
        element = d.createElement(name)
        text = d.createTextNode(value)
        element.appendChild(text)
        musicvideoelement.appendChild(element)

    title = os.path.basename(video)
    title, _ = os.path.splitext(title)

    # YouTube IDs.
    title = re.sub("-[a-zA-Z0-9-_]{11}$", "", title)

    # Dunders.
    if "_-_" in title:
        title = title.replace("_", " ")

    pathcomponents = video.split("/")
    artist = None
    album = None
    if "Albums and singles" in pathcomponents:
        spoint = pathcomponents.index("Albums and singles")
        if pathcomponents[spoint + 2] == os.path.basename(video):
            artist = pathcomponents[spoint + 1]
        elif pathcomponents[spoint + 3] == os.path.basename(video):
            artist = pathcomponents[spoint + 1]
            album = pathcomponents[spoint + 2]

    if artist:
        # Remove artist from title.
        artistre = re.escape(artist.lower())
        featurings = [
            "&",
            "vs\\.",
            "ft\\.",
            "vs",
            "ft",
            "and",
            "feat\\.",
            "featuring",
            "feat",
            "pres\\.",
            "presenting",
            "pres",
        ]
        additionals_post = "|".join(" +%s .+?" % x for x in featurings)
        additionals_pre = "|".join(".+? %s +" % x for x in featurings)
        artistres = [
            (
                "(((%s|)%s)( - | -|- | ' | '))" % (additionals_pre, artistre),
                2,
            ),
            (
                "((%s(|%s))( - | -|- | ' | '))" % (artistre, additionals_post),
                2,
            ),
            (
                "( - | -|- )(((%s|)%s))" % (additionals_pre, artistre),
                2,
            ),
            (
                "( - | -|- )((%s(|%s)))" % (artistre, additionals_post),
                2,
            ),
            #      '(\\([^()_]*?%s[^()_]*?\\))'%artistre,
            #      '(\\[[^()_]*?%s[^()_]*?\\])'%artistre,
        ]
        regexmatch = None
        for regex, group in artistres:
            regex = re.compile(regex, re.IGNORECASE)
            regexmatch = regex.search(title)
            if regexmatch:
                title = title.replace(regexmatch.group(0), "")
                artist = regexmatch.group(group)
                break

        # take care of the "rsnetini tnsrei trnietn (feat samantha fox)" names
        additionals_feat = "|".join("%s [^)-]+" % x for x in featurings)
        featres = [
            (" +\\((%s)\\)" % additionals_feat, 1),
            (
                " +(%s)" % additionals_feat,
                1,
            ),
        ]
        for regex, group in featres:
            regex = re.compile(regex, re.IGNORECASE)
            regexmatch = regex.search(title)
            if regexmatch:
                title = title.replace(regexmatch.group(0), "")
                artist = artist + " " + regexmatch.group(1)
                break

        setElement("artist", artist)
    if album:
        setElement("album", album)
    if title:
        setElement("title", title)

    text = d.toprettyxml()[23:].encode("utf-8")
    if os.path.exists(nfo):
        with open(nfo, "rb") as f:
            originaltext = f.read()
        if originaltext == text:
            return
    print("Updated NFO for %s" % video)
    print("Contents:\n%s" % text.decode("utf-8"))
    if not dryrun:
        with open(nfo, "wb") as f:
            f.write(text)


E_FAILURE = "failure"
E_INTR = "interrupted"


def thread_main(video: str, nfo: str, dryrun=False) -> str:
    try:
        donfo(video, nfo, dryrun)
    except KeyboardInterrupt:
        if os.path.exists(nfo) and not dryrun:
            os.unlink(nfo)
        return E_INTR
    except BaseException as e:
        print("Failure processing", video, "\n", "    ->", e, file=sys.stderr)
        if os.path.exists(nfo) and not dryrun:
            os.unlink(nfo)
        return E_FAILURE


def main(args: typing.Iterable[str]) -> int:
    dryrun = False
    if args[0] == "-n":
        dryrun = True
        args = args[1:]
    basedir = args[0]
    iterable = find_videos(basedir)
    for video, nfofile in iterable:
        ret = thread_main(video, nfofile, dryrun)
        if ret == E_FAILURE:
            return 8
        if ret == E_INTR:
            print("interrupted", sys.stderr)
            return 4
    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))
